module type T = sig
  type elt 
  type t 
  val empty: t
  val add: elt -> t -> t
  val elems: t -> elt list
  val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
end

module Make (E : Set.OrderedType) = struct
  module Elt = E
  type elt = E.t
  type t = { elems: elt list; }
  let empty = { elems = [] }
  let add q s = { elems = q :: s.elems }
  let elems s = s.elems
  let fold f s z = List.fold_left (fun z e -> f e z) z s.elems
end

module Product
  (S1: T)
  (S2: T) =
struct
  module E = struct type t = S1.elt * S2.elt let compare = compare end
  module R = Make(E)
  include R
  let product s1 s2 =  (* Q = Q1 x Q2 = { (q1,q2) | q1 in Q1, q2 in Q2 } *)
    let f x y t = R.add (x,y) t in
    let g x t = S2.fold (f x) s2 t in
    S1.fold g s1 R.empty
end


