module type Attr = sig type t end

module type T = sig
  type elt 
  type attr
  type t 
  module S: Myset.T with type elt = elt
  val empty: t
  val add: elt * attr -> t -> t
  val elems: t -> (elt * attr) list
  val set_of: t -> S.t
  val fold: (elt * attr -> 'a -> 'a) -> t -> 'a -> 'a
end

module Make (E : Set.OrderedType) (A : Attr) = struct
  module Elt = E
  type elt = E.t
  type attr = A.t
  module S = Myset.Make(E)
  type t = { elems: S.t; attrs: (elt * attr) list }
  let empty = { elems = S.empty; attrs = [] }
  let add (e,a) s = { elems = S.add e s.elems; attrs = (e,a) :: s.attrs }
  let elems s = s.attrs
  let set_of s = s.elems
  let fold f s z = List.fold_left (fun z e -> f e z) z s.attrs
end

module Product
  (S1: T)
  (S2: T) =
struct
  module P = Myset.Product(S1.S)(S2.S)
  module R = Make (P.E) (struct type t = S1.attr * S2.attr end)
  include R
  let product s1 s2 =
    { elems = P.product (S1.set_of s1) (S2.set_of s2);
      attrs =
        let f (e1,a1) acc (e2,a2) = ((e1,e2),(a1,a2))::acc in
        let g acc x = List.fold_left (f x) acc (S2.elems s2) in
        List.fold_left g [] (S1.elems s1) }
end


